@@ -1,11 +1,13 @@
 # -*- coding:utf-8 -*-
 import re
 import urllib
+from HTMLParser import HTMLParser
 from urlparse import urlparse, parse_qs
 
 from pyquery import PyQuery
 import lxml.html
-from yandex_parser.exceptions import EmptySerp, YandexParserError, YandexParserContentError
+from yandex_parser.exceptions import EmptySerp, YandexParserError, YandexParserContentError, YandexParserContextError, \
+    YandexParserBadContentError
 from yandex_parser.utils import to_unicode, get_full_domain_without_scheme
 from lxml import etree
 
@@ -47,7 +49,7 @@ class YandexParser(object):
         return {'u': res.group(1), 't': YandexParser.strip_tags(res.group(2))}
 
     def get_context_visible_url(self, sn):
-        els = sn.xpath('.//div[contains(@class,"typo_type_greenurl")]/div/a')
+        els = sn.xpath('.//div[contains(@class,"Typo_type_greenurl")]/div/a')
         if not els:
             return ''
         return unicode(els[0].text_content())
@@ -62,7 +64,13 @@ class YandexParser(object):
 
         return 'b'
 
+    def raise_on_bad_content(self):
+        if '<title>400</title>' in self.content:
+            raise YandexParserBadContentError
+
     def get_context_serp(self):
+        self.raise_on_bad_content()
+
         # очищаем встроенные в рекламу сниппеты serp-item
         content = re.sub(
             ur'<div class=direct-map-modal__snippet><div class=serp-item.*?</div>\s*</div>',
@@ -72,29 +80,33 @@ class YandexParser(object):
         dom = PyQuery(content)
         serp = dom('.serp-item')
 
-        r_blocks, tb_blocks = self._find_context_r_or_tb_blocks(serp)
-        tb_blocks = self._aggregate_organic_blocks(tb_blocks)
-        tb_blocks = self._remove_little_organic_blocks(tb_blocks)
-        t_blocks, b_blocks = self._divide_context_tb_blocks(tb_blocks)
+        t_blocks, b_blocks, r_blocks = self._find_context_r_or_tb_blocks(serp)
 
         self._set_a(t_blocks, 't')
         self._set_a(b_blocks, 'b')
         self._set_a(r_blocks, 'r')
 
+        if len(t_blocks) > 4:
+            raise YandexParserContextError('too many top-blocks')
+
+        if len(b_blocks) > 6:
+            raise YandexParserContextError('too many bottom-blocks')
+
         result = self._format_context_blocks(t_blocks + b_blocks + r_blocks)
+
         return {'pc': len(result), 'sn': result}
 
     def _find_context_r_or_tb_blocks(self, serp):
-        tb_blocks = []
+        t_blocks = []
+        b_blocks = []
         r_blocks = []
-        organic_block_len = 0
+
+        is_organic_block = False
         for index, sn in enumerate(serp):
             is_ignore_block = self._ignore_block(sn)
-            is_context_snippet = self._is_context_snippet(sn)
 
             if not is_ignore_block:
-                organic_block_len += 1
-                tb_blocks.append({'index': index, 'sn': self.CONTEXT_ORGANIC_BLOCK})
+                is_organic_block = True
                 continue
 
             # исключаем блок директа с баннером
@@ -115,94 +127,15 @@ class YandexParser(object):
             if self._is_card_narrow(sn):
                 continue
 
-            if not is_context_snippet:
+            if not self._is_context_snippet(sn):
                 continue
 
-            t_or_b = sn.xpath('.//div[contains(@class,"typo_type_greenurl")]/div[contains(@class,"label_color_yellow")]') \
-                or sn.cssselect('li.serp-adv-item div.organic__subtitle') \
-                or sn.xpath('.//div[contains(@class,"typo_type_greenurl")]/div[contains(@class,"label_theme_direct")]')
-            if t_or_b:
-                tb_blocks.append({'index': index, 'sn': sn})
-                continue
-            r_blocks.append({'index': index, 'sn': sn})
-        return r_blocks, tb_blocks
+            if not is_organic_block:
+                t_blocks.append({'index': index, 'sn': sn})
+            else:
+                b_blocks.append({'index': index, 'sn': sn})
 
-    def _remove_little_organic_blocks(self, tb_blocks):
-        max_ob_len = 0
-        max_ob_index = None
-        for i, block in enumerate(tb_blocks):
-            if block['sn'] == self.CONTEXT_ORGANIC_BLOCK and block['len'] >= max_ob_len:
-                max_ob_len = block['len']
-                max_ob_index = i
-
-        if max_ob_index is None:
-            return tb_blocks
-
-        blocks = []
-        for i, block in enumerate(tb_blocks):
-            if block['sn'] != self.CONTEXT_ORGANIC_BLOCK:
-                blocks.append(block)
-
-            if i != max_ob_index:
-                continue
-
-            blocks.append(block)
-        return blocks
-
-    def _aggregate_organic_blocks(self, tb_blocks):
-        agg_blocks = []
-        ob_len = 0
-        old_ob_block = None
-        for block in tb_blocks:
-            if block['sn'] != self.CONTEXT_ORGANIC_BLOCK:
-                if ob_len:
-                    old_ob_block['len'] = ob_len
-                    agg_blocks.append(old_ob_block)
-                    old_ob_block = None
-                    ob_len = 0
-                agg_blocks.append(block)
-                continue
-
-            old_ob_block = block
-            ob_len += 1
-        if ob_len:
-            old_ob_block['len'] = ob_len
-            agg_blocks.append(old_ob_block)
-        return agg_blocks
-
-    def _divide_context_tb_blocks(self, tb_blocks):
-        t_blocks = []
-        b_blocks = []
-        is_ob_exists = filter(lambda x: x['sn'] == self.CONTEXT_ORGANIC_BLOCK, tb_blocks)
-        if is_ob_exists:
-            is_bottom = False
-            for block in tb_blocks:
-                if block['sn'] == self.CONTEXT_ORGANIC_BLOCK:
-                    is_bottom = True
-                    continue
-
-                if is_bottom:
-                    b_blocks.append(block)
-                    continue
-
-                t_blocks.append(block)
-        else:
-            is_bottom = False
-            old_class = None
-            for block in tb_blocks:
-                sn = block['sn']
-                cur_class = filter(lambda x: re.match(ur'data-[\w\d]{4,}', x), sn.attrib)[0]
-                if old_class and cur_class != old_class:
-                    is_bottom = True
-                old_class = cur_class
-
-                if is_bottom:
-                    b_blocks.append(block)
-                    continue
-
-                t_blocks.append(block)
-
-        return t_blocks, b_blocks
+        return t_blocks, b_blocks, r_blocks
 
     def _format_context_blocks(self, blocks):
         result = []
@@ -211,7 +144,7 @@ class YandexParser(object):
             title, url = self._get_title(sn, False)
 
             if not re.search('^https?://yabs\.yandex\.ru', url):
-                raise YandexParserError(u'incorrect context url={}'.format(url))
+                raise YandexParserContextError(u'incorrect context url={}'.format(url))
 
             result.append({
                 'u': url,
@@ -318,6 +251,8 @@ class YandexParser(object):
         return int(match.group(1))
 
     def get_serp(self):
+        self.raise_on_bad_content()
+
         if self.is_not_found():
             return {'pc': 0, 'sn': []}
 
@@ -420,8 +355,26 @@ class YandexParser(object):
         return url, infected
 
     def _get_title(self, sn, infected):
-        is_video_snippet = 't-construct-adapter__free-video' in sn.attrib['class']
+        link = sn.cssselect('a.OrganicTitle-Link')
+        if link:
+            url = link[0].attrib['href']
 
+            title_element = link[0].cssselect('h2.OrganicTitle-LinkText')
+            if not title_element:
+                title_element = link[0].cssselect('div.OrganicTitle-LinkText')
+                title_element_text = title_element[0].cssselect('div.Organic-Title')
+                if title_element_text:
+                    title_element = title_element_text
+            if not title_element:
+                title_element = link[0]
+
+            if not title_element:
+                raise YandexParserError('not found title text')
+
+            title = unicode(title_element[0].text_content())
+            return title, url
+
+        is_video_snippet = 't-construct-adapter__free-video' in sn.attrib['class']
         h2 = self._get_title_h2(sn, is_video_snippet)
         if not h2:
             return None, self._get_url_from_mobile_greenurl(sn)
@@ -481,7 +434,16 @@ class YandexParser(object):
         if sn.xpath('.//div[contains(@class,"label_theme_direct")]'):
             return True
 
-        return 'serp-adv' in sn.attrib['class'] or 't-construct-adapter__adv' in sn.attrib['class']
+        if 'serp-adv' in sn.attrib['class'] or 't-construct-adapter__adv' in sn.attrib['class']:
+            return True
+
+        html = etree.tostring(sn, method='html')
+        html = HTMLParser().unescape(html)
+
+        if re.search('<span class="[^"]+?">реклама', html, flags=re.I):
+            return True
+
+        return 'Реклама<span aria-hidden="true">' in html
 
     def _is_card_narrow(self, sn):
         # боковая карта справа
@@ -501,6 +463,11 @@ class YandexParser(object):
         if self._is_card_narrow(sn):
             return True
 
+        # Войдите, чтобы пользоваться
+        if 'zalogin-smart' in sn.attrib['class']:
+            return True
+
+
         # игнорим новости
         if 't-construct-adapter__news' in sn.attrib['class']:
             return True
@@ -646,6 +613,13 @@ class YandexParser(object):
             return True
 
         html = etree.tostring(sn, method='html', encoding='UTF-8')
+        # предложения магазинов
+        if 'ecommerce_carousel' in html:
+            return True
+
+        if '<div class="ECommerceCarousel' in html:
+            return True
+
         if 't-construct-adapter__market' in sn.attrib['class']:
             if re.search(ur'<div class="organic typo typo_text_m typo_line_s">\s*<div class="organic__content-wrapper clearfix">', html, re.I | re.M):
                 return True
@@ -669,6 +643,9 @@ class YandexParser(object):
         if 'calculator__wrapper' in html:
             return True
 
+        if '<div class="ProductGallery' in html:
+            return True
+
         # чаты с организациями
         if 'chat-list__header' in html:
             return True
@@ -736,6 +713,9 @@ class YandexParser(object):
                     if title is None and url is None:
                         continue
 
+                    if re.search('^https?://yabs\.yandex\.ru', url):
+                        continue
+
                 except YandexParserError:
                     if is_composite_gap:
                         continue
@@ -852,7 +832,7 @@ class YandexParser(object):
 
         captcha_type = None
         form_method = None
-        if re.search(ur'>Я не робот<', self.content):
+        if re.search(ur'CheckboxCaptcha-Button', self.content):
             url_image = None
             captcha_type = 'i_not_robot'
 
